@@ -1,6 +1,6 @@
 package Munin::Master::ProcessManager;
 
-# $Id: ProcessManager.pm 4169 2011-04-19 03:01:40Z bldewolf $
+# $Id: ProcessManager.pm 4524 2011-12-20 11:36:25Z steve.schnepp $
 
 use warnings;
 use strict;
@@ -10,6 +10,7 @@ use English qw(-no_match_vars);
 use IO::Socket;
 use POSIX qw(:sys_wait_h);
 use Storable qw(nstore_fd fd_retrieve);
+use Time::HiRes;
 
 use Munin::Common::Timeout;
 use Munin::Master::Config;
@@ -73,6 +74,7 @@ sub start_work {
     
     do_with_timeout($self->{timeout}, sub {
         $self->_collect_results($sock);
+	return 1;
     }) or croak "Work timed out before all workers finished";
 
     $self->{workers} = [];
@@ -111,6 +113,9 @@ sub _start_waiting_workers {
         last if scalar keys %{$self->{active_workers}} == $self->{max_concurrent};
 	# Here we just start'em, check results in _collect_results
         $self->_start_next_worker();
+    } continue {
+	    # Sleep for a random time, to avoid too much process creation all at once
+	    sleep ( rand ($config->{worker_start_delay}) ) if $config->{worker_start_delay};
     }
 }
 
@@ -157,6 +162,7 @@ sub _collect_results {
 
         my $timed_out = !do_with_timeout($self->{accept_timeout}, sub {
             accept $worker_sock, $sock;
+	    return 1;
         });
         if ($timed_out) {
 	    if (defined($self->{host})) {
@@ -245,6 +251,7 @@ sub _do_work {
     eval {
         my $timed_out = !do_with_timeout($self->{worker_timeout}, sub {
             $res = $worker->do_work();
+	    return 1;
         });
 	if (!defined($res)) {
 	    ERROR "[ERROR] $worker failed to connect to node";
