@@ -1,7 +1,7 @@
 package Munin::Master::Utils;
 # -*- cperl -*-
 
-# $Id: Utils.pm 4169 2011-04-19 03:01:40Z bldewolf $
+# $Id: Utils.pm 4445 2011-10-18 08:07:12Z steve.schnepp $
 
 use strict;
 use warnings;
@@ -16,27 +16,35 @@ use Munin::Common::Defaults;
 use Munin::Master::Logger;
 use Munin::Master::Config;
 use Munin::Common::Config;
-use Log::Log4perl qw (:easy);
+use Log::Log4perl qw(:easy);
 use POSIX qw(strftime);
-use POSIX ":sys_wait_h";
-use RRDs;
-use Symbol 'gensym';
+use POSIX qw(:sys_wait_h);
+use Symbol qw(gensym);
+use Data::Dumper;
+use Storable;
+use Scalar::Util qw(isweak weaken);
 
 our (@ISA, @EXPORT);
 
 @ISA = ('Exporter');
-@EXPORT = ('munin_trend',
-	   'munin_fetch',
+@EXPORT = (
 	   'munin_nscasend',
 	   'munin_createlock',
 	   'munin_removelock',
 	   'munin_runlock',
 	   'munin_getlock',
 	   'munin_readconfig',
+	   'munin_readconfig_storable',
 	   'munin_writeconfig',
+	   'munin_writeconfig_storable',
+	   'munin_read_storable',
+	   'munin_write_storable',
 	   'munin_delete',
 	   'munin_overwrite',
+	   'munin_dumpconfig',
+	   'munin_dumpconfig_as_str',
 	   'munin_config',
+	   'munin_refreshconfig',
 	   'munin_draw_field',
 	   'munin_get_bool',
 	   'munin_get_bool_val',
@@ -46,6 +54,7 @@ our (@ISA, @EXPORT);
 	   'munin_node_status',
 	   'munin_category_status',
 	   'munin_get_picture_filename',
+	   'munin_get_picture_loc',
 	   'munin_get_html_filename',
 	   'munin_get_filename',
 	   'munin_get_keypath',
@@ -54,10 +63,14 @@ our (@ISA, @EXPORT);
 	   'munin_get_field_order',
 	   'munin_get_rrd_filename',
 	   'munin_get_node_name',
+	   'munin_get_orig_node_name',
 	   'munin_get_parent_name',
+	   'munin_get_node_fqn',
+	   'munin_find_node_by_fqn',
 	   'munin_get_node_loc',
 	   'munin_get_node',
 	   'munin_set_var_loc',
+	   'munin_set_var_path',
 	   'munin_set',
 	   'munin_copy_node_toloc',
 	   'munin_get_separated_node',
@@ -68,10 +81,12 @@ our (@ISA, @EXPORT);
 	   'munin_get_children',
 	   'munin_get_node_partialpath',
 	   'munin_has_subservices',
+	   'munin_get_host_path_from_string',
 	   'print_version_and_exit',
 	   'exit_if_run_by_super_user',
 	   'look_for_child',
 	   'wait_for_remaining_children',
+	   'auto_weaken',
 	   );
 
 my $VERSION = $Munin::Common::Defaults::MUNIN_VERSION;
@@ -88,26 +103,59 @@ my @dircomponents = split('/',$0);
 my $me = pop(@dircomponents);
 
 
-sub munin_trend {
-    my (@array) = @_;
-    return ($array[$#array] - $array[0]);
+# `auto_weaken` performs a breadth-first descend of all strong references to
+# references (possible cycles). If it encounters a reference to an item it has
+# already seen earlier it will weaken that reference.
+sub auto_weaken {
+	my $items = 0;
+	my $weakened = 0;
+	my %seen = map{ $_ => 1 } @_;
+	my @todo = @_;
+	while (my $cur = shift @todo) {
+		$items++;
+		if (ref($cur) eq 'HASH') {
+			for my $key (keys %$cur) {
+				next unless ref $cur->{$key};
+				next if isweak $cur->{$key};
+				my $tgt = $cur->{$key};
+				if ($seen{$tgt}) {
+					$weakened++;
+					weaken $cur->{$key};
+				} else {
+					$seen{$tgt} = 1;
+					push @todo, $tgt;
+				}
+			}
+		} elsif (ref($cur) eq 'ARRAY') {
+			for (my $i = 0 ; $i < @$cur ; $i++) {
+				next unless ref $cur->[$i];
+				next if isweak $cur->[$i];
+				my $tgt = $cur->[$i];
+				if ($seen{$tgt}) {
+					$weakened++;
+					weaken $cur->[$i];
+				} else {
+					$seen{$tgt} = 1;
+					push @todo, $tgt;
+				}
+			}
+		} elsif (ref($cur) eq 'SCALAR') {
+			next unless ref $$cur;
+			next if isweak $cur;
+			my $tgt = $$cur;
+			if ($seen{$tgt}) {
+				$weakened++;
+				weaken $cur;
+			} else {
+				$seen{$tgt} = 1;
+				push @todo, $tgt;
+			}
+		}
+	}
+	# print "items: $items, weakened: $weakened\n";
+	return @_;
 }
 
-
-sub munin_fetch {
-    my ($file,$last,$type) = @_;
-    my ($start,$step,$names,$data) = RRDs::fetch $file,$type || "AVERAGE";
-    unless (defined $data)
-    {
-        WARN ("[WARNING] Could not fetch data from $file(".($type||"AVERAGE")."): ". RRDs::error);
-        return;
-    }
-    my @array = map { @$_[0] } splice(@$data, $#$data - ($last || 1));
-    return $array[0] if (!$last);
-    return @array;
-}
-
-
 sub munin_draw_field {
     my $hash   = shift;
 
@@ -231,11 +279,11 @@ sub munin_overwrite {
 
     my ($configfile,$overwrite) = @_;
     for my $key (keys %$overwrite) {
-        next if $key =~ /^#%#/;
+	next if substr($key,0,3) eq '#%#';
 	if (ref $overwrite->{$key}) {
 	    if (!defined $configfile->{$key}) {
 		if (ref $overwrite->{$key} eq "HASH") {
-		    $configfile->{$key}->{'#%#parent'} = $configfile;
+		    $configfile->{$key}->{'#%#parent'} = $configfile; weaken($configfile->{$key}->{'#%#parent'});
 		    $configfile->{$key}->{'#%#name'}   = $key;
 		    munin_overwrite($configfile->{$key},$overwrite->{$key});
 		} else {
@@ -251,22 +299,41 @@ sub munin_overwrite {
     return ($configfile);
 }
 
+sub munin_readconfig_storable {
+	my ($conf) = @_;
+	
+	my $config = undef;
+	$conf ||= $configfile;
+
+    # try to read storable version
+    if ( (-r $conf) && open (my $CFG_STORABLE, '<', $conf)) { 
+		DEBUG "[DEBUG] munin_readconfig: found Storable version of $conf, using it";
+        $config = Storable::fd_retrieve($CFG_STORABLE); 
+        close ($CFG_STORABLE); 
+	}
+
+	return $config; 
+}
 
 sub munin_readconfig {
     my ($conf, $missingok, $corruptok) = @_;
 
     my $config = undef;
-    my @contents = undef;
 
     $conf ||= $configfile;
-    if (! -r $conf and ! $missingok) {
-	WARN "munin_readconfig: cannot open '$conf'";
-	return;
-    }
-    if (open (my $CFG, '<', $conf)) {
-	@contents = <$CFG>;
-	close ($CFG);
-        $config = munin_parse_config (\@contents);
+
+    # try first to read storable version
+	$config = munin_readconfig_storable("$conf.storable");
+	if(!defined $config){
+        if (! -r $conf and ! $missingok) {
+            WARN "munin_readconfig: cannot open '$conf'";
+            return;
+		}
+        if (open (my $CFG, '<', $conf)) {
+            my @contents = <$CFG>;
+            close ($CFG);
+            $config = munin_parse_config (\@contents);
+        }
     }
 
     # Some important defaults before we return...
@@ -275,10 +342,11 @@ sub munin_readconfig {
     $config->{'dbdir'}         ||= $Munin::Common::Defaults::MUNIN_DBDIR;
     $config->{'logdir'}        ||= $Munin::Common::Defaults::MUNIN_LOGDIR;
     $config->{'tmpldir'}       ||= "$Munin::Common::Defaults::MUNIN_CONFDIR/templates/";
+    $config->{'staticdir'}       ||= "$Munin::Common::Defaults::MUNIN_CONFDIR/static/";
     $config->{'htmldir'}       ||= $Munin::Common::Defaults::MUNIN_HTMLDIR;
-    $config->{'spooldir'}      ||= $Munin::Common::Defaults::MUNIN_SSPOOLDIR;
+    $config->{'spooldir'}      ||= $Munin::Common::Defaults::MUNIN_SPOOLDIR;
     $config->{'#%#parent'}     = undef;
-    $config->{'#%#name'}       = "#%#root";
+    $config->{'#%#name'}       = "root";
 
     return ($config);
 }
@@ -397,28 +465,28 @@ sub munin_get_var_path
     return;
 }
 
- 
+
 sub munin_find_field {
     # Starting at the (presumably the root) $hash make recursive calls
     # until for example graph_title or value is found, and then
-    # continue recursing and itterating to all are found.
+    # continue recursing and itterating until all are found.
     #
     # Then we return a array of pointers into the $hash
+    #
+    # This function will not use REs as they are inordinately
+    # expensive.  There is a munin_find_field_for_limits that will
+    # match REs instead of whole strings.
+
+    my ($hash, $field, $avoid) = @_;
 
-    my $hash  = shift;
-    my $field = shift;
-    my $avoid = shift;
     my $res = [];
 
-    if (ref ($field) ne "Regexp") {
-	$field = qr/^$field$/;
-    }
-
     if (ref ($hash) eq "HASH") {
 	foreach my $key (keys %{$hash}) {
-	    next if $key =~ /^#%#/;
+	    next if substr($key,0,3) eq '#%#';
 	    last if defined $avoid and $key eq $avoid;
-	    if ($key =~ $field) {
+	    # Always check $key eq $field first here, or we break
+	    if ($key eq $field) {
 		push @$res, $hash;
 	    } elsif (ref ($hash->{$key}) eq "HASH") {
 		push @$res, @{munin_find_field ($hash->{$key}, $field, $avoid)};
@@ -431,9 +499,8 @@ sub munin_find_field {
 
 
 sub munin_find_field_for_limits {
-    my $hash  = shift;
-    my $field = shift;
-    my $avoid = shift;
+    my ($hash, $field, $avoid) = @_;
+
     my $res = [];
 
     if (ref ($field) ne "Regexp") {
@@ -442,7 +509,7 @@ sub munin_find_field_for_limits {
 
     if (ref ($hash) eq "HASH") {
 	foreach my $key (keys %{$hash}) {
-	    next if $key =~ /^#%#/;
+	    next if substr($key,0,3) eq '#%#';
 	    last if defined $avoid and $key eq $avoid;
 	    if (ref ($hash->{$key}) eq "HASH") {
 		push @$res, @{munin_find_field_for_limits ($hash->{$key}, $field, $avoid)};
@@ -463,7 +530,7 @@ sub munin_get_children {
     return if (ref ($hash) ne "HASH");
 
     foreach my $key (keys %{$hash}) {
-	next if $key =~ /^#%#/;
+	next if substr($key,0,3) eq '#%#';
 	if (defined $hash->{$key} and ref ($hash->{$key}) eq "HASH") {
 	    push @$res, $hash->{$key};
 	}
@@ -480,7 +547,7 @@ sub munin_get_separated_node
 
     if (ref ($hash) eq "HASH") {
 	foreach my $key (keys %$hash) {
-	    next if $key =~ /^#%#/;
+	    next if substr($key,0,3) eq '#%#';
 	    if (ref ($hash->{$key}) eq "HASH") {
 		$ret->{$key} = munin_get_separated_node ($hash->{$key});
 	    } else {
@@ -506,19 +573,68 @@ sub munin_get_parent_name
     }
 }
 
+sub munin_get_orig_node_name {
+    my $hash = shift;
+
+    if (ref ($hash) eq "HASH" and defined $hash->{'#%#name'}) {
+		return (defined $hash->{'#%#origname'}) ? $hash->{'#%#origname'} : $hash->{'#%#name'};
+    } else { 
+	return;
+    }
+}
 
 sub munin_get_node_name
 {
     my $hash = shift;
 
     if (ref ($hash) eq "HASH" and defined $hash->{'#%#name'}) {
-	return $hash->{'#%#name'};
+		return $hash->{'#%#name'};
     } else { 
 	return;
     }
 }
 
 
+sub munin_get_node_fqn
+{
+    my $hash = shift;
+
+    if (ref ($hash) eq "HASH") {
+	my $fqn = "";
+	if (defined $hash->{'#%#name'}) {
+		$fqn = $hash->{'#%#name'};
+	}
+ 	if (defined $hash->{'#%#parent'}) {
+		# Recursively prepend the parent, concatenation with /
+		$fqn = munin_get_node_fqn ($hash->{'#%#parent'}) . "/" . $fqn;
+	}
+	return $fqn;
+    } else {
+	return;
+    }
+}
+
+
+sub munin_find_node_by_fqn {
+    # The FQN should be relative to the point in the hash we're handed.
+    my($hash,$fqn) = @_;
+
+    my $here = $hash;
+
+    my @path = split('/',$fqn);
+
+    foreach my $pc (@path) {
+	next if $pc eq 'root';
+	if (exists $here->{$pc}) {
+	    $here = $here->{$pc};
+	} else {
+	    confess "Could not find FQN $fqn!";
+	}
+    }
+    return $here ;
+}
+
+
 sub munin_get_picture_loc {
     my $hash = shift;
     my $res = [];
@@ -528,9 +644,12 @@ sub munin_get_picture_loc {
     }
     if (defined $hash->{'#%#origin'}) {
 	$res = munin_get_picture_loc ($hash->{'#%#origin'});
+    } elsif (defined $hash->{'#%#origparent'}){
+        $res = munin_get_picture_loc ($hash->{'#%#origparent'});
+        push @$res, munin_get_orig_node_name ($hash) if defined $res;
     } elsif (defined $hash->{'#%#parent'}) {
-	$res = munin_get_picture_loc ($hash->{'#%#parent'});
-	push @$res, munin_get_node_name ($hash) if defined $res;
+	    $res = munin_get_picture_loc ($hash->{'#%#parent'});
+	    push @$res, munin_get_orig_node_name ($hash) if defined $res;
     }
     return $res;
 }
@@ -541,16 +660,19 @@ sub munin_get_node_loc {
     my $res = [];
 
     if (ref ($hash) ne "HASH") { # Not a has node
-    	return;
+        return;
     }
     if (defined $hash->{'#%#parent'}) {
-	$res = munin_get_node_loc ($hash->{'#%#parent'});
-	push @$res, munin_get_node_name ($hash) if defined $res;
+        if(defined $hash->{'#%#origparent'}){
+            $res = munin_get_node_loc ($hash->{'#%#origparent'});
+        } else {
+            $res = munin_get_node_loc ($hash->{'#%#parent'});
+        }
+        push @$res, munin_get_orig_node_name ($hash) if defined $res;
     }
     return $res;
 }
 
-
 sub munin_get_parent {
     my $hash = shift;
 
@@ -574,11 +696,13 @@ sub munin_get_node {
     my $loc  = shift;
 
     foreach my $tmpvar (@$loc) {
-	if ($tmpvar !~ /\S/) {
-	    ERROR "[ERROR] munin_get_node: Cannot work on hash node \"$tmpvar\"";
+	if (! exists $hash->{$tmpvar} ) {
+	    # Only complain on a blank key if is no key like that. Usually it
+	    # shouln't, so it avoids a needless regexp in this highly used
+	    # function
+	    ERROR "[ERROR] munin_get_node: Cannot work on hash node '$tmpvar'" if ($tmpvar !~ /\S/);
 	    return undef;
-	}
-	return undef if !exists $hash->{$tmpvar};
+        }	
 	$hash = $hash->{$tmpvar};
     }
     return $hash;
@@ -598,20 +722,31 @@ sub munin_set_var_loc
     my $hash = shift;
     my $loc  = shift;
     my $val  = shift;
-    my @aloc = @$loc;
 
-    my $tmpvar = shift @aloc;
-    $tmpvar = shift @aloc while (defined $tmpvar and $tmpvar =~ /^#%#/);
-    if ($tmpvar !~ /\S/) {
+    my $iloc = 0;
+
+    # XXX -  Dirty breaking recursive function 
+    # --> Using goto is BAD, but enough for now
+START:
+
+    # Find the next normal value (that doesn't begin with #%#)
+    my $tmpvar = $loc->[$iloc++];
+    $tmpvar = $loc->[$iloc++] while (defined $tmpvar and
+				 substr($tmpvar,0,3) eq '#%#');
+
+    if (index($tmpvar, " ") != -1) {
 	ERROR "[ERROR] munin_set_var_loc: Cannot work on hash node \"$tmpvar\"";
 	return;
     }
-    if (@aloc > 0) {
-	if (!defined $hash->{$tmpvar} or !defined $hash->{$tmpvar}->{"#%#name"}) { # Init the new node
-	    $hash->{$tmpvar}->{"#%#parent"} = $hash;
+    if (scalar @$loc > $iloc) {
+	if (!defined $hash->{$tmpvar} or !defined $hash->{$tmpvar}->{"#%#name"}) { 
+            # Init the new node
+	    $hash->{$tmpvar}->{"#%#parent"} = $hash; weaken($hash->{$tmpvar}->{"#%#parent"});
 	    $hash->{$tmpvar}->{"#%#name"} = $tmpvar;
 	}
-        return munin_set_var_loc ($hash->{$tmpvar}, \@aloc, $val);
+	# Recurse
+        $hash = $hash->{$tmpvar};
+	goto START;
     } else {
         WARN "[WARNING] munin_set_var_loc: Setting unknown option '$tmpvar' at "
 	    . munin_get_keypath($hash)
@@ -643,9 +778,9 @@ sub munin_get_node_partialpath
 	my @leftarr = split (/;/, $leftstring);
 	my @rightarr = split (/\./, $rightstring);
 	push @$varloc, @leftarr, @rightarr
-    } elsif ($var =~ /^\s*([^;:\.]+)\s*$/) {
+    } elsif ($var =~ /^\s*([^;:.]+)\s*$/) {
 	push @$varloc, $var;
-    } elsif ($var =~ /^\s*(.+)\.([^\.:;]+)$/) {
+    } elsif ($var =~ /^\s*([^.]+)\.([^:;]+)$/) {
 	my ($leftstring, $rightstring) = ($1, $2);
 
 	my @leftarr = split (/;/, $leftstring);
@@ -728,7 +863,7 @@ sub munin_writeconfig_loop {
     my ($hash,$fh,$pre) = @_;
 
     foreach my $key (keys %$hash) {
-	next if $key =~ /#%#/;
+	next if substr($key,0,3) eq '#%#';
 	my $path = (defined $pre ? join(';', ($pre, $key)) : $key);
 	if (ref ($hash->{$key}) eq "HASH") {
 	    munin_writeconfig_loop ($hash->{$key}, $fh, $path);
@@ -748,28 +883,100 @@ sub munin_writeconfig_loop {
     }
 }
 
+sub munin_read_storable {
+	my ($storable_filename, $default) = @_;
+
+	return Storable::retrieve($storable_filename) if (-e $storable_filename);
+	
+	# Default
+	return $default;
+}
+
+sub munin_write_storable {
+	my ($storable_filename, $data) = @_;
+
+	my $storable_filename_tmp = $storable_filename . ".tmp.$$";
+
+	# Write datafile.storable, in network order to be architecture indep
+        Storable::nstore($data, $storable_filename_tmp);
+
+	# Atomic commit
+	rename ($storable_filename_tmp, $storable_filename);
+}
+
+sub munin_writeconfig_storable {
+	my ($datafilename,$data) = @_;
+
+	DEBUG "[DEBUG] Writing state to $datafilename";
+
+	# XXX - Refs should be weakened on construction
+	auto_weaken($data);
+
+	munin_write_storable($datafilename, $data);
+}
 
 sub munin_writeconfig {
     my ($datafilename,$data,$fh) = @_;
 
     DEBUG "[DEBUG] Writing state to $datafilename";
 
-    if (!defined $fh) {
+    my $is_fh_already_managed = defined $fh;
+    if (! $is_fh_already_managed) {
 	$fh = gensym();
 	unless (open ($fh, ">", $datafilename)) {
 	    LOGCROAK "Fatal error: Could not open \"$datafilename\" for writing: $!";
 	}
     }
-
+    
     # Write version
     print $fh "version $VERSION\n";
     # Write datafile
     munin_writeconfig_loop ($data, $fh);
-
-    DEBUG "[DEBUG] Closing filehandle \"$datafilename\"...\n";
-    close ($fh);
+    
+    if (! $is_fh_already_managed) {
+        DEBUG "[DEBUG] Closing filehandle \"$datafilename\"...\n";
+        close ($fh);
+    }
 }
 
+sub munin_dumpconfig_as_str {
+    my ($config) = @_;
+
+    local $Data::Dumper::Sortkeys = sub { [ sort grep {!/^#%#parent$/} keys %{$_[0]} ]; };
+    local $Data::Dumper::Indent = 1;
+
+    return Dumper $config;
+}
+
+
+sub munin_dumpconfig {
+    my ($config) = @_;
+    my $indent = $Data::Dumper::Indent;
+    my $sorter = $Data::Dumper::Sortkeys;
+
+    $Data::Dumper::Sortkeys =
+      sub { [ sort grep {!/^#%#parent$/} keys %{$_[0]} ]; };
+    $Data::Dumper::Indent = 1;
+
+    print "##\n";
+    print "## NOTE: #%#parent is hidden to make the print readable!\n";
+    print "##\n";
+    print Dumper $config;
+
+    $Data::Dumper::Sortkeys = $sorter;
+    $Data::Dumper::Indent = $indent;
+}
+
+sub munin_refreshconfig {
+    my $config   = shift;
+    my @stat = stat("$config->{dbdir}/datafile");
+    if ($config->{'#%#datafile_mtime'} && $stat[9] > $config->{'#%#datafile_mtime'}) {
+	$config = munin_readconfig("$config->{dbdir}/datafile");
+    }
+
+    $config->{'#%#datafile_mtime'} = $stat[9];
+    return $config;
+}
 
 sub munin_config {
     my $conffile = shift;
@@ -825,13 +1032,13 @@ sub munin_get_html_filename {
     } @$loc;
 	
     if (defined $hash->{'graph_title'} and !munin_has_subservices ($hash)) {
-	$plugin = pop @$loc or return;
+        $plugin = pop @$loc or return;
     }
-
+    
     if (@$loc) { # The rest is used as directory names...
-	$ret .= "/" . join ('/', @$loc);
+        $ret .= "/" . join ('/', @$loc);
     }
-
+    
     return "$ret/$plugin.html";
 }
 
@@ -921,7 +1128,7 @@ sub munin_get_keypath {
 	# Not sure when a #%#name node can go missing
 	my $name = $i->{'#%#name'} || '*BUG*';
 	goto gotoparent if $name eq '*BUG*';
-	last if $name eq '#%#root';
+	last if $name eq 'root';
 	if ($host) {
 	    # Into group land now
 	    unshift(@group,$name);
@@ -1040,14 +1247,20 @@ sub munin_get_bool_val
 
 sub munin_get
 {
-    my $hash   = shift;
-    my $field  = shift;
-    my $default = shift;
+    my ($hash, $field, $default) = @_;
 
-    return $default if (ref ($hash) ne "HASH");
-    return $hash->{$field} if defined $hash->{$field} and ref($hash->{$field}) ne "HASH";
-    return $default if not defined $hash->{'#%#parent'};
-    return munin_get ($hash->{'#%#parent'}, $field, $default);
+    # Iterate to top if needed
+    do {
+        return $default if (ref ($hash) ne "HASH");
+
+        my $hash_field = $hash->{$field};
+        return $hash_field if (defined $hash_field && ref($hash_field) ne "HASH");
+
+        # Go up
+        $hash = $hash->{'#%#parent'};
+    } while (defined $hash);
+
+    return $default;
 }
 
 
@@ -1061,7 +1274,7 @@ sub munin_copy_node_toloc
 
     if (ref ($from) eq "HASH") {
 	foreach my $key (keys %$from) {
-	    next if $key =~ /^#%#/;
+	    next if substr($key,0,3) eq '#%#';
 	    if (ref ($from->{$key}) eq "HASH") {
 		munin_copy_node_toloc ($from->{$key}, $to, [@$loc, $key]);
 	    } else {
@@ -1267,7 +1480,7 @@ sub munin_get_field_order
     for my $fieldnode (@{munin_find_field ($hash, "label")}) {
         my $fieldname = munin_get_node_name ($fieldnode);
 	push @$result,$fieldname
-	    if !grep m[^\Q$fieldname\E(?:=|$)], @$result;;
+	    if !grep m[^\Q$fieldname\E(?:=|$)], @$result;
     }
 
     for my $fieldnode (@{munin_find_field ($hash, "stack")}) {
@@ -1300,7 +1513,6 @@ sub munin_get_rrd_filename {
     my $path    = shift;
 
     my $result  = undef;
-    my $name    = munin_get_node_name ($field);
 
     # Bail out on bad input data
     return if !defined $field or ref ($field) ne "HASH";
@@ -1415,6 +1627,21 @@ sub munin_has_subservices {
     return $hash->{'#%#has_subservices'};
 }
 
+sub munin_get_host_path_from_string {
+	# splits a host definition, as used in the config, into a group array and a host name
+	my $key = shift;
+	my (@groups) = split(';', $key);
+	my $host = pop(@groups);
+	if(scalar @groups > 0){
+	} else {
+		my @groups = split('.', $key);
+		if(scalar @groups > 1){
+			@groups = ($groups[0]);
+		}
+	}
+	return (\@groups, $host);
+}
+
 1;
 
 __END__
@@ -1496,9 +1723,6 @@ Returns:
  - Success: Boolean; true if field will be graphed, false if not
  - Failure: undef
 
-=item B<munin_fetch>
-
-
 
 =item B<munin_field_status>
 
@@ -1711,7 +1935,7 @@ Returns:
 =item B<munin_get_picture_loc>
 
 Get location array for hash node for picture purposes. Differs from
-munin_get_node_loc in that it honors #%#origin metadata 
+munin_get_node_loc in that it honors #%#origin metadata
 
 Parameters: 
  - $hash: A ref to the node 
@@ -1892,10 +2116,6 @@ Returns:
  - Failure: The $hash we were handed
 
 
-=item B<munin_trend>
-
-
-
 =item B<munin_writeconfig>
 
 
