@@ -1,4 +1,5 @@
 # -*- cperl -*-
+# vim: ts=4 : sw=4 : et
 #
 # Copyright (C) 2004-2009 Dagfinn Ilmari Mannsaaker, Nicolai Langfeldt,
 # Linpro AS
@@ -24,7 +25,7 @@
 # pattern.  Please maintain it in the same way.
 
 
-# $Id: SNMP.pm 3868 2010-08-19 09:19:37Z mha $
+# $Id: SNMP.pm 3887 2010-09-14 12:23:01Z ligne $
 
 =head1 NAME
 
@@ -63,7 +64,7 @@ use Munin::Plugin;
 our (@ISA, $DEBUG);
 
 @ISA = qw(Net::SNMP);
-  
+
 # Alias $Munin::Plugin::SNMP::DEBUG to $Munin::Plugin::DEBUG, so SNMP
 # plugins don't need to import the latter module to get debug output.
 *DEBUG = \$Munin::Plugin::DEBUG;
@@ -163,7 +164,7 @@ The following environment variables are consulted:
 
 =over
 
-=item env.host
+=item C<env.host>
 
 If the plugin name (symlink) does not contain the host name this is
 used as the host name to connect to.
@@ -182,19 +183,19 @@ hosts file) it is possible to do this:
   [snmp_switch2.langfeldt.net]
      env.host 192.168.2.46
 
-=item env.port
+=item C<env.port>
 
 The port to connect to.  Default 161.
 
-=item env.timeout
+=item C<env.timeout>
 
 The timeout in seconds to use. Default 5.
 
-=item env.version
+=item C<env.version>
 
 The SNMP version to use for the connection. One of 1, 2, 3, snmpv1,
 snmpv2c or snmpv3.  SNMP v2 is better as it supports bulk operations.
-Therefore 2 is the default in Munin::Plugin::SNMP.  If your device
+Therefore 2 is the default in C<Munin::Plugin::SNMP>.  If your device
 supports v3 that may be even better as it supports proper security -
 but the encryption may slow things down.
 
@@ -205,26 +206,25 @@ Security is handled differently for versions 1/2c and 3.  See below.
     my ($host, $port, $version, $tail) = config_session();
 
     # Common options.
-    my @options = (-hostname    => $host,
-	           -port	=> $port,
-                   -version     => $version,
+    my @options = (
+        -hostname => $host,
+        -port     => $port,
+        -version  => $version,
     );
 
-    
     # User defined options
-    if (defined($userargs[0])) {
-	push(@options,@userargs);
-    }
-    
-    # Timeout 
-    if (defined($ENV{timeout})) {
-	push(@options, (-timeout => $ENV{timeout}));
-    }
-    
-    if ($version eq '1' or $version eq 'snmpv1' or
-	$version eq '2' or $version eq 'snmpv2c') {
+    push @options, @userargs;
 
-=item env.community
+    # Timeout
+    push @options, (-timeout => $ENV{timeout}) if $ENV{timeout};
+
+    if ($version eq '1' or $version eq 'snmpv1'
+     or $version eq '2' or $version eq 'snmpv2c')
+    {
+
+=item SNMP 1/2c authentication
+
+=item C<env.community>
 
 The community name for version 1 and 2c agents. The default is
 'public'.  If this works your device is probably very insecure and
@@ -232,87 +232,80 @@ needs a security checkup.
 
 =cut
 
-	# FIXME: die if $ENV{community} isn't set?
-	my $community = $ENV{community} || 'public';
+        # FIXME: die if $ENV{community} isn't set?
+        my $community = $ENV{community} || 'public';
 
-	push(@options,(-community => $community));
+        push @options, (
+            -community => $community
+        );
 
-	my $object;
-	my $error;
+        print STDERR "Setting up a SNMPv$version session\n" if $DEBUG;
 
-	print STDERR "Setting up a SNMPv$version session\n" if $DEBUG;
+        my ($object, $error) = $class->SUPER::session(@options);
+        die "Could not set up SNMP $version session to $host: $error\n" unless $object;
 
-	($object, $error) = $class->SUPER::session(@options);
-
-	unless ($object) {
-	    die "Could not set up SNMP $version session to $host: $error\n";
-	}
-
-	return $object;
+        return $object;
 
     } elsif ($version eq '3' or $version eq 'snmpv3') {
 
 =item SNMP 3 authentication
 
-SNMP v3 has three security levels: "noAuthNoPriv".  If a username
-and password is given it goes up to "authNoPriv".
+SNMP v3 has three security levels. Lowest is C<noAuthNoPriv>, which
+provides neither authentication nor encryption.  If a username and
+C<authpassword> are given it goes up to C<authNoPriv>, and the
+connection is authenticated.  If C<privpassword> is given the security
+level becomes C<authPriv> - the connection is authenticated and
+encrypted.
 
-If privpassword is given the security level becomes "authPriv" - the
-connection is authenticated and encrypted.
-
-Note: Encryption can slow down slow or heavily loaded network devices.
-For most uses authNoPriv will be secure enough --- in SNMP v3 the
+B<Note>: Encryption can slow down slow or heavily loaded network
+devices.  For most uses C<authNoPriv> will be secure enough -- the
 password is sent over the network encrypted in any case.
 
-Munin::Plugin::SNMP does not support ContextEngineIDs and such for
+C<Munin::Plugin::SNMP> does not support ContextEngineIDs and such for
 authentication/privacy.  If you see the need and know how it should be
 done please send patches!
 
 For further reading on SNMP v3 security models please consult RFC3414
-and the documentation for Net::SNMP.
+and the documentation for L<Net::SNMP>.
 
-If version is set to 3 or snmpv3 these variables are used to define
-authentication:
+If version is set to 3 or snmpv3 the following variables are used to
+define authentication:
 
 =over
 
-=item env.v3username
+=item C<env.v3username>
 
-SNMPv3 username.  There is no default. Empty username ('') is allowed.
+Username.  There is no default.
 
-=item env.v3authpassword
-
-SNMPv3 authentication password.  Optional when encryption is also
-enabled, in which case defaults to the privacy password.
-Authentication requires a v3authprotocol, but this defaults to "md5"
-and may therefore be left unspecified.
+=item C<env.v3authpassword>
 
+Authentication password.  Optional when encryption is also enabled, in
+which case defaults to the privacy password (C<env.v3privpassword>).
 The password is sent encrypted (one way hash) over the network.
 
-=item env.v3authprotocol
+=item C<env.v3authprotocol>
 
-SNMPv3 authentication protocol.  One of 'md5' or 'sha' (HMAC-MD5-96,
-RFC1321 and SHA-1/HMAC-SHA-96, NIST FIPS PIB 180, RFC2264).  The
-default is 'md5'.
+Authentication protocol.  One of 'md5' or 'sha' (HMAC-MD5-96, RFC1321
+and SHA-1/HMAC-SHA-96, NIST FIPS PIB 180, RFC2264).  The default is
+'md5'.
 
-=item env.v3privpassword
+=item C<env.v3privpassword>
 
-SNMPv3 privacy password to enable encryption.  An empty ('') password
-is considered as no password and will not enable encryption.
+Privacy password to enable encryption.  An empty ('') password is
+considered as no password and will not enable encryption.
 
 Privacy requires a v3privprotocol as well as a v3authprotocol and a
-v3authpassword, but all of these are defaulted (to 'des', 'md5', and
-the v3privpassword value, respectively) and may therefore be left
+v3authpassword, but all of these are defaulted (to 'des', 'md5', and the
+v3privpassword value, respectively) and may therefore be left
 unspecified.
 
-=item env.v3privprotocol
+=item C<env.v3privprotocol>
 
 If the v3privpassword is set this setting controls what kind of
-encryption is used to achive privacy in the session.  Only the very
-weak 'des' encryption method is supported officially.  The default is
-'des'.
+encryption is used to achive privacy in the session.  Only the very weak
+'des' encryption method is supported officially.  The default is 'des'.
 
-The implementing perl module (Net::SNMP) also supports '3des'
+The implementing perl module (L<Net::SNMP>) also supports '3des'
 (CBC-3DES-EDE aka Triple-DES, NIST FIPS 46-3) as specified in IETF
 draft-reeder-snmpv3-usm-3desede.  Whether or not this works with any
 particular device, we do not know.
@@ -320,45 +313,50 @@ particular device, we do not know.
 =back
 
 =cut
-	
-	my $privpw    = $ENV{'v3privpassword'} || '';
-	my $privproto = $ENV{'v3privprotocol'} || 'des';
 
-	my $authpw    = $ENV{'v3authpassword'} || '';
-	my $authproto = $ENV{'v3authprotocol'} || 'md5';
-	my $username  = $ENV{'v3username'};
+        # This would be much tidier if Perl had switch statements with
+        # fall-through :-)
 
-	if (defined($username)) {
-            # FIXME: isn't it an error if no username was specified?
-	    push( @options, (-username => $username));
-	}
+        # Privacy
+        my $privpw    = $ENV{'v3privpassword'};
+        my $privproto = $ENV{'v3privprotocol'} || 'des';
 
-	if ($privpw) {
-	    # Privacy is a stronger demand and should be checked first.
-	    push( @options, ( -privpassword => $privpw,
-			      -privprotocol => $privproto,
-			      -authpassword => ($authpw || $privpw),
-			      -authprotocol => $authproto ));
+        if ($privpw) {
+            push @options, (
+                -privpassword => $privpw,
+                -privprotocol => $privproto,
+            );
+            print STDERR "Enabled SNMPv3 privacy.\n" if $DEBUG;
+        }
 
-	    # Note how Net::SNMP demands authentication options when
-	    # privacy is invoked.
-	} elsif ($authpw) {
-	    # Authenticated only.
-	    push( @options,
-		  ( -authpassword => $authpw,
-		    -authprotocol => $authproto ));
-	}
+        # Authentication.  Password defaults to env.v3privpassword
+        my $authpw    = $ENV{'v3authpassword'} || $privpw;
+        my $authproto = $ENV{'v3authprotocol'} || 'md5';
 
-	my ($object, $error) = $class->SUPER::session(@options);
+        if ($authpw) {
+            push @options, (
+                -authpassword => $authpw,
+                -authprotocol => $authproto,
+            );
+            print STDERR "Enabled SNMPv3 authentication.\n" if $DEBUG;
+        }
 
-	if (!defined($object)) {
-	    die "Could not set up SNMPv3 session to $host: $error\n";
-	}
+        # Username
+        my $username  = $ENV{'v3username'};
 
-	return $object;
+        if ($username) {
+            push @options, (
+                -username => $username
+            );
+        }
 
-    } else {
-	die "Unknown SNMP version: '$version'.";
+        my ($object, $error) = $class->SUPER::session(@options);
+        die "Could not set up SNMPv3 session to $host: $error\n" unless $object;
+
+        return $object;
+    }
+    else {
+        die "Unknown SNMP version: '$version'.";
     }
 }
 
